From 8ed60795a7c278f9699616f72eb05ce49800ba6f Mon Sep 17 00:00:00 2001
From: zj <1772600164@qq.com>
Date: Wed, 07 Aug 2024 01:20:50 +0800
Subject: [PATCH] 1
---
bitgetsClient/src/main/java/org/example/bitgetsclient/wsClient/BitgetClient.java | 129 +++++++++++++++++++++++++++----------------
1 files changed, 81 insertions(+), 48 deletions(-)
diff --git a/bitgetsClient/src/main/java/org/example/bitgetsclient/wsClient/BitgetClient.java b/bitgetsClient/src/main/java/org/example/bitgetsclient/wsClient/BitgetClient.java
index b9c61a8..c3cb229 100644
--- a/bitgetsClient/src/main/java/org/example/bitgetsclient/wsClient/BitgetClient.java
+++ b/bitgetsClient/src/main/java/org/example/bitgetsclient/wsClient/BitgetClient.java
@@ -8,13 +8,22 @@
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;
import lombok.extern.slf4j.Slf4j;
+import org.example.bitgetsclient.BitgetsClientApplication;
+import org.example.bitgetsclient.comm.ApplicationContextProvider;
import org.example.bitgetsclient.util.RedisUtil;
import org.json.JSONException;
+import org.springframework.beans.factory.annotation.Autowired;
+import org.springframework.boot.SpringApplication;
+import org.springframework.context.ApplicationContext;
+import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.annotation.Bean;
+import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import javax.websocket.*;
import java.io.IOException;
+import java.lang.reflect.Type;
+import java.math.BigDecimal;
import java.net.URI;
import java.nio.ByteBuffer;
import java.util.Arrays;
@@ -24,6 +33,7 @@
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
+import java.util.concurrent.atomic.AtomicInteger;
@ClientEndpoint
@Slf4j
@@ -53,7 +63,7 @@
- public void start() {
+ public void start() throws Exception {
try {
connect(); // 尝试连接
if (session == null) {
@@ -65,7 +75,6 @@
executorService.scheduleAtFixedRate(this::sendPing, PING_INTERVAL, PING_INTERVAL, TimeUnit.MILLISECONDS);
// 发送订阅消息
session.getBasicRemote().sendText(subscriptions); // 发送订阅信息
-
synchronized (this) {
this.wait(); // 等待 WebSocket 消息到来
}
@@ -75,6 +84,7 @@
log.error("线程被中断: " + e.getMessage(), e); // 记录线程中断的错误
} catch (Exception e) {
log.error("bitget ws 连接过程中发生异常: " + e.getMessage(), e); // 记录连接过程中发生的异常
+ throw e;
} finally {
executorService.shutdownNow(); // 尝试立即关闭调度服务
}
@@ -105,17 +115,39 @@
// 确保 dataNode 不为 null,以避免空指针异常
if (dataNode != null) {
- // 存储数据到 HashMap
- Map<String, Object> bidAskMap = new HashMap<>(); // 变量命名更具描述性
- bidAskMap.put("bids", dataNode.get("bids")); // 获取并存储 bids
- bidAskMap.put("asks", dataNode.get("asks")); // 获取并存储 asks
+ Map<String, Object> hashMap = new HashMap<>(); // 变量命名更具描述性
- Map<String, Object> argMap = gson.fromJson(map.get("arg").toString(), new TypeToken<Map<String, Object>>() {}.getType()); // 解析 arg 为 Map
- String key = "bitget" + argMap.get("instId"); // 构建 Redis 存储的键
+ String asks = dataNode.get("bids").toString();
+ String bids = dataNode.get("asks").toString();
- // 存储到 Redis,使用 ObjectMapper 转换为 JSON 字符串
- String jsonData = objectMapper.writeValueAsString(bidAskMap); // 先将 HashMap 转换为 JSON 字符串
- RedisUtil.set(key, jsonData); // 存储数据
+ Type listType = new TypeToken<List<List<String>>>(){}.getType();
+ List<List<String>> bidsList = gson.fromJson(bids, listType);
+ List<List<String>> asksList = gson.fromJson(asks, listType);
+
+ if(!bidsList.isEmpty() && !bidsList.get(0).isEmpty() && !asksList.isEmpty() && !asksList.get(0).isEmpty()){
+ if (!bidsList.isEmpty() && !bidsList.get(0).isEmpty()) {
+ List<String> bidsStringList = bidsList.get(0);
+ HashMap<String, Object> pvMap = new HashMap<>(); // 创建新的 HashMap 保存 bids 和 asks
+ pvMap.put("p", new BigDecimal(bidsStringList.get(0)).toPlainString());
+ pvMap.put("v", new BigDecimal(bidsStringList.get(1)).toPlainString());
+ hashMap.put("bids", pvMap); // 获取并存储 bids
+ }
+
+ if (!asksList.isEmpty() && !asksList.get(0).isEmpty()) {
+ List<String> asksStringList = asksList.get(0); // 修正此处为 asksList
+ HashMap<String, Object> pvMap = new HashMap<>(); // 创建新的 HashMap 保存 bids 和 asks
+ pvMap.put("p", new BigDecimal(asksStringList.get(0)).toPlainString());
+ pvMap.put("v", new BigDecimal(asksStringList.get(1)).toPlainString());
+ hashMap.put("asks", pvMap); // 获取并存储 asks
+ }
+
+ Map<String, Object> argMap = gson.fromJson(map.get("arg").toString(), new TypeToken<Map<String, Object>>() {}.getType()); // 解析 arg 为 Map
+ String key = "bitget" + argMap.get("instId"); // 构建 Redis 存储的键
+
+ // 存储到 Redis,使用 ObjectMapper 转换为 JSON 字符串
+ String jsonData = objectMapper.writeValueAsString(hashMap); // 先将 HashMap 转换为 JSON 字符串
+ RedisUtil.set(key, jsonData); // 存储数据
+ }
}
}
} catch (JsonSyntaxException e) {
@@ -143,51 +175,52 @@
}
@OnClose
- public void onClose() {
+ public void onClose() throws Exception {
log.info("bitget ws 连接已关闭,尝试重新连接..."); // 记录连接关闭的信息
- handleConnectionClosedOrError(); // 处理连接关闭事件
+ throw new Exception();
}
@OnError
- public void onError(Throwable throwable) {
+ public void onError(Throwable throwable) throws Exception {
log.error("bitget ws 发生错误: " + throwable.getMessage(), throwable); // 记录错误信息
- handleConnectionClosedOrError(); // 处理错误事件
+ throw new Exception();
}
- private void handleConnectionClosedOrError() {
- synchronized (lock) {
- if (!reconnecting) {
- reconnecting = true; // 开始重连
- executorService.execute(this::attemptReconnect); // 执行重连操作
- }
- }
- }
+// private void handleConnectionClosedOrError() {
+// executorService.execute(() -> {
+// try {
+// attemptReconnect();
+// } catch (Exception e) {
+// throw new RuntimeException(e);
+// }
+// });
+// }
- private void attemptReconnect() {
- if (reconnectAttempts < MAX_RECONNECT_ATTEMPTS) { // 检查重连次数是否超过限制
- try {
- log.info("bitget ws 开始重连"); // 记录开始重连的信息
- connect(); // 尝试重连
- log.info("bitget ws 重连成功"); // 成功重连的日志
- reconnectAttempts = 0; // 重连成功,重置重连次数
- } catch (Exception e) {
- reconnectAttempts++; // 增加重连尝试次数
- log.warn("bitget ws 重连失败,尝试次数: " + reconnectAttempts, e); // 记录重连失败的警告信息
- // 采用指数退避策略,增加重连间隔
- long waitTime = Math.min(60, (long) Math.pow(2, reconnectAttempts)) * 1000; // 最大等待时间为 60 秒
- scheduleReconnect(waitTime); // 调度下次重连
- }
- } else {
- log.error("超过最大重连次数,停止重连"); // 超过最大重连次数
- reconnecting = false; // 重连状态重置
- }
- }
-
- private void scheduleReconnect(long waitTime) { // 接受等待时间参数
- if (!executorService.isShutdown()) {
- executorService.schedule(this::attemptReconnect, waitTime, TimeUnit.MILLISECONDS); // 调度重连
- }
- }
+// private void attemptReconnect() throws Exception {
+// if (reconnectAttempts < MAX_RECONNECT_ATTEMPTS) { // 检查重连次数是否超过限制
+// try {
+// log.info("bitget ws 开始重连"); // 记录开始重连的信息
+// connect(); // 尝试重连
+// log.info("bitget ws 重连成功"); // 成功重连的日志
+// reconnectAttempts = 0; // 重连成功,重置重连次数
+// } catch (Exception e) {
+// reconnectAttempts++; // 增加重连尝试次数
+// log.warn("bitget ws 重连失败,尝试次数: " + reconnectAttempts, e); // 记录重连失败的警告信息
+// // 采用指数退避策略,增加重连间隔
+// long waitTime = Math.min(60, (long) Math.pow(2, reconnectAttempts)) * 1000; // 最大等待时间为 60 秒
+// scheduleReconnect(waitTime); // 调度下次重连
+// }
+// } else {
+// log.error("超过最大重连次数,停止重连"); // 超过最大重连次数
+// throw new Exception();
+// }
+// }
+//
+// private void scheduleReconnect(long waitTime) { // 接受等待时间参数
+// if (!executorService.isShutdown()) {
+// executorService.schedule(this::attemptReconnect, waitTime, TimeUnit.MILLISECONDS); // 调度重连
+// }
+// }
private void sendPing() {
try {
--
Gitblit v1.9.3